<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Nov 26, 2019
 */

/**
 * Nodeworx Controller for Acronis integration
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Nodeworx_Acronis extends Ctrl_Nodeworx_Plugin
{
    private $_Plugin;

    const ACTIVE_TAB = 'active_tab';
    const IS_AUTHORIZED = 'authorized';
    const INSTALL_BACKUP_AGENT = 'install_backup_agent';

    const AB_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const AB_MESSAGE_API_FAILURE = '##LG_ACRONIS_API_FAILURE##';
    const AB_MESSAGE_INSTALLATION_ERROR = '##LG_ACRONIS_INSTALLATION_ERROR##';
    const AB_MESSAGE_INSTALL_SUCCESS = '##LG_ACRONIS_INSTALL_SUCCESS##';
    const AB_MESSAGE_AGENT_ALREADY_INSTALLED = '##LG_ACRONIS_AGENT_ALREADY_INSTALLED##';
    const AB_MESSAGE_NO_BACKUP_AGENT = '##LG_ACRONIS_NO_BACKUP_AGENT##';
    const AB_MESSAGE_UNINSTALL_ERROR = '##LG_ACRONIS_UNINSTALL_ERROR##';
    const AB_MESSAGE_UNINSTALL_SUCCESS = '##LG_ACRONIS_UNINSTALL_SUCCESS##';


    /**
     * block actions if not master nodeworx user.
     *
     * @param string $action
     * @return boolean
     */
    public function isActionAllowed($action)
    {
        if (IW::NW()->isReseller() || !$this->_Plugin->isCurrentNwUserAllowed()) {
            $this->_blockAction('##LG_ACTION_NOT_PERMITTED##');
            return false;
        }

        if (Ini::get(Ini::IWORX, 'goiworx') == '0') {
            return true;
        }

        $parent_check = parent::isActionAllowed($action);
        if ($parent_check === true) {
            if (IW::NW()->isReseller() === true) {
                if ($this->_isAllowedAsReseller($action) === false) {
                    $this->_blockAction('##LG_ACTION_NOT_PERMITTED##');
                    return false;
                }
            }
        }
        return $parent_check;
    }

    /**
     * Is allowed as reseller.
     *
     * @param string $action
     * @return boolean
     */
    protected function _isAllowedAsReseller($action)
    {
        return false;
    }

    /**
     * Init function. Final construct will run this method.
     */
    protected function _init()
    {
        $this->_Plugin = $this->_getPlugin();
    }


    /**
     * Index action.
     */
    public function indexAction()
    {
        $isAuthorized = $this->_Plugin->userIsAuthorized();

        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, $isAuthorized);
        $View->assign('nw_user', true);

        if (!$isAuthorized) {
            $this->_displayFailure('_notAuthorized', self::AB_MESSAGE_NOT_AUTHORIZED);
        }


        $BackupAgent = $this->_Plugin->getBackupAgent();

        $backupAgentNotInstalled = !$BackupAgent->isInstalled();
        $scriptsMissing = $BackupAgent->scriptsMissing();
        if ($isAuthorized && ($backupAgentNotInstalled || $scriptsMissing)) {
            $View->assign(self::INSTALL_BACKUP_AGENT, true);
        }
    }

    /**
     * Exists because index action can't refer to itself
     * when displayFailure is triggered.
     */
    private function _notAuthorizedAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, false);
    }

    /**
     * Install backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function installBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $this->_installAgentOrAddScripts();
            $this->_displaySuccess(self::AB_MESSAGE_INSTALL_SUCCESS);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_INSTALLATION_ERROR);
        }
    }

    /**
     * Install backup agent or copy pre/post scripts to acronis location.
     */
    private function _installAgentOrAddScripts()
    {
        $BackupAgent = $this->_Plugin->getBackupAgent();

        if ($BackupAgent->isInstalled()) {
            $BackupAgent->addPrePostScriptsIfMissing();
        }

        if (!$BackupAgent->isInstalled()) {
            $BackupAgent->install();
        }
    }

    /**
     * Uninstall backup agent.
     *
     * @param Form_SubmitOnly $Form
     */
    public function uninstallBackupAgentCommitAction(Form_SubmitOnly $Form)
    {
        $BackupAgent = $this->_Plugin->getBackupAgent();

        if (!$BackupAgent->isInstalled()) {
            $this->_displayFailure('index', self::AB_MESSAGE_NO_BACKUP_AGENT);
        }

        try {
            $BackupAgent->uninstall();
            $this->_displaySuccess(self::AB_MESSAGE_UNINSTALL_SUCCESS);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_UNINSTALL_ERROR);
        }
    }

    /**
     * Get cloud storage usage data.
     *
     * @param Form_SubmitOnly $Form
     */
    public function cloudStorageCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $CloudStorage = $this->_Plugin->getCloudStorage();
            $storageData = $CloudStorage->getStorageData();
            $Payload = new Payload($storageData);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }


    /**
     * Get backup plan status (enabled/disabled) and backup plan schedule.
     *
     * @param Form_SubmitOnly $Form
     */
    public function backupPlanStatusCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $CloudStorage = $this->_Plugin->getCloudStorage();
            $payloadData = array();

            if ($CloudStorage->isProtected()) {
                $BackupPlan = $this->_getBackupPlan();
                $payloadData = $BackupPlan->getPlanDetails();
            }

            $Payload = new Payload($payloadData);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get backup plan details.
     *
     * @return Model_BackupPlan_BackupPlan
     */
    private function _getBackupPlan()
    {
        $planId = $this->_getExistingPlanId();
        $BackupPlan = $this->_Plugin->getBackupPlan($planId);
        return $BackupPlan;
    }

    /**
     * Get existing plan id.
     *
     * @return string
     */
    private function _getExistingPlanId()
    {
        $Resource = $this->_Plugin->getResource();
        return $Resource->getPlanId();
    }


    /**
     * Display last backup status, date, and time.
     *
     * @param Form_SubmitOnly $Form
     */
    public function backupInfoCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $backupInfo = $this->_getBackupStatusFromResource();
            $Payload = new Payload($backupInfo);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }


    /**
     * Get storage data.
     *
     * @return array
     */
    private function _getBackupStatusFromResource()
    {
        $CloudStorage = $this->_Plugin->getCloudStorage();

        // only display last backup & status if next backup is scheduled.
        if ($CloudStorage->isProtected()) {
            $Resource = $this->_Plugin->getResource();
            $backupStatus = $Resource->getBackupStatus();
            return $backupStatus;
        } else {
            return array();
        }
    }

    /**
     * Get redirect URL for one-time login. Link is valid for 30 seconds.
     *
     * @param Form_SubmitOnly $Form
     */
    public function redirectToBackupCloudCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $loginLink = $this->_getLoginLink();
            $Payload = new Payload($loginLink);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get login link.
     *
     * @return array
     */
    private function _getLoginLink()
    {
        $Auth = $this->_Plugin->getAuth();
        $loginLink = $Auth->getOneTimeLoginLink();
        return array('redirect' => $loginLink);
    }

    /**
     * Revoke backup plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function revokePlanCommitAction(Form_SubmitOnly $Form)
    {

        try {
            $this->_removeResourceFromPlan();
            $this->_displayPayload(new Payload('Backup plan was successfully revoked.'));
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Revoke plan.
     */
    private function _removeResourceFromPlan()
    {
        $BackupPlan = $this->_getBackupPlan();
        $instanceId = $this->_getInstanceId();

        $BackupPlan->removeResource($instanceId);
    }

    /**
     * Get resource id.
     *
     * @return string
     */
    private function _getInstanceId()
    {
        $Resource = $this->_Plugin->getResource();
        return $Resource->getInstanceId();
    }

    /**
     * Activate backup plan.
     *
     * @param Form_SubmitOnly $Form
     */
    public function activatePlanCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $BackupPlan = $this->_createNewPlanIfNeeded();
            $this->_awaitNextBackup();

            $payloadData = $BackupPlan->getPlanDetails();
            $Payload = new Payload($payloadData);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Attempt to return existing plan. Create new if needed.
     *
     * @return Model_BackupPlan_BackupPlan
     */
    private function _createNewPlanIfNeeded()
    {
        try {
            $planId = $this->_getExistingPlanId();
            $BackupPlan = $this->_Plugin->getBackupPlan($planId);
            $instanceId = $this->_getInstanceId();
            $BackupPlan->addResource($instanceId);
            return $BackupPlan;
        } catch (Exception $e) {
            $newPlan = $this->_getNewPlan();
            return $newPlan;
        }
    }

    /**
     * Create new plan & return id.
     *
     * @return string
     */
    private function _getNewPlan()
    {
        $Draft = $this->_Plugin->getPlanDraft();
        $Draft->createPlan();
        $planId = $Draft->getPlanId();
        return $this->_Plugin->getBackupPlan($planId);
    }

    /**
     * Await backup status change to 'running'.
     */
    private function _awaitNextBackup()
    {
        $Resource = $this->_Plugin->getResource();
        $attempts = 0;

        while ($Resource->backupNotScheduled() && $attempts < 10) {
            sleep(2);
            $attempts++;
        }
    }

    /**
     * Run backup.
     *
     * @param Form_SubmitOnly $Form
     */
    public function runBackupCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $this->_runBackup();
            $this->_awaitResourceBackupStatus();

            $backupInfo = $this->_getBackupStatusFromResource();
            $Payload = new Payload($backupInfo);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Run backup.
     */
    private function _runBackup()
    {
        $instanceId = $this->_getInstanceId();
        $BackupPlan = $this->_getBackupPlan();
        $BackupPlan->runBackup($instanceId);
    }

    /**
     * Await backup status change to 'running'.
     */
    private function _awaitResourceBackupStatus()
    {
        $Resource = $this->_Plugin->getResource();
        $attempts = 0;

        while (!$Resource->backupInProgress() && $attempts < 10) {
            sleep(2);
            $attempts++;
        }
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listBackupsCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $backups = $this->_getBackups();
            $this->_displayPayloadAsJson($backups);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get backups.
     *
     * @return array
     */
    private function _getBackups()
    {
        $backups = array();

        $CloudStorage = $this->_Plugin->getCloudStorage();
        if ($CloudStorage->isProtected()) {
            $Resource = $this->_Plugin->getResource();
            $backups = $Resource->getBackups();
            $returnValue = array_map(function ($backup) {
                return array(
                    'backupId' => $backup['backupId'],
                    'date' => $backup['date'],
                    'created' => $backup['created']);
            }, $backups);
        }

        return $returnValue;
    }

    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    public function getRootContentCommitAction(Form_AcronisBackups $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $RecoveryPoint = $this->_Plugin->getRecoveryPoint($backupId);
            $dirContent = $RecoveryPoint->getRootContent();
            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    public function getDirectoryContentCommitAction(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $dirId = $Form->getDirectoryId();
            $RecoveryPoint = $this->_Plugin->getRecoveryPoint($backupId);
            $dirContent = $RecoveryPoint->getDirectoryContent($dirId);
            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Display json if cli
     *
     * @param mixed $payloadContent
     */
    private function _displayPayloadAsJson($payloadContent)
    {
        // render as json
        if (IW::Env()->matches(IW::CLI)) {
            IW::Env()->setOutputStyle('pretty');
        }

        $Payload = new Payload($payloadContent);
        $this->_displayPayload($Payload);
    }

    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    public function recoverDirectoriesCommitAction(Form_AcronisBackups_RecoverDirectories $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $dirIds = $Form->getDirectoryIds();
            $shouldDeleteOlderFiles = $Form->deleteRequested();

            $RecoveryPlan = $this->_Plugin->getRecoveryPlan($backupId);
            $RecoveryPlan->recoverDirectories($dirIds, $shouldDeleteOlderFiles);


            $Payload = new Payload('Directories successfully recovered');
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get restore location.
     */
    public function getRestoreAltLocationAction()
    {
        $altLocation = Model_RecoveryPlan_RecoveryPlan::getNwRestoreAltLocationPath();
        $Payload = new Payload(array(
           'altLocation' => $altLocation
        ));

        $this->_displayPayload($Payload);
    }



    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    public function restoreToAltLocationCommitAction(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $dirIds = $Form->getDirectoryIds();

            $RecoveryPlan = $this->_Plugin->getRecoveryPlan($backupId);
            $RecoveryPlan->restoreToAltLocation($dirIds);

            $Payload = new Payload('restore complete');
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Log error msg.
     *
     * @param string    $functionName
     * @param Exception $e
     */
    private function _logError($functionName, Exception $e)
    {
        $exceptionDetails = $this->_getMessageFromException($e);
        IW::getLog()->error("Action failed $functionName: $exceptionDetails");
    }

    /**
     * Get message from exception
     *
     * @param Exception $e
     */
    private function _getMessageFromException(Exception $e)
    {
        $class = $e->getFile();
        $line = $e->getLine();
        $message = $e->getMessage();
        return "$message. File: $class, line: $line";
    }
}
