<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Jun 12, 2020
 */

/**
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class Ctrl_Siteworx_Acronis extends Ctrl_Siteworx_Plugin
{
    const IS_AUTHORIZED = 'authorized';
    const CONTACT_ADMIN = 'display_contact_admin';
    const AB_MESSAGE_API_FAILURE = '##LG_ACRONIS_API_FAILURE##';
    const AB_MESSAGE_NOT_AUTHORIZED = '##LG_ACRONIS_NOT_AUTHORIZED##';
    const ACTION_NOT_ALLOWED = '##LG_ACTION_BLOCKED##';
    const RESTORE_COMPLETE_ALT_LOCATION = '##LG_RESTORE_ALT_LOCATION_COMPLETE##';
    const DIRECTORIES_RECOVERED= '##LG_RESTORE_DIRECTORIES_COMPLETE##';


    /**
     * Init function. Final construct will run this method.
     */
    protected function _init()
    {
        $this->_Plugin = $this->_getPlugin();
    }

    /**
     * Allow actions if master user.
     *
     * @param string $action
     */
    public function isActionAllowed($action)
    {
        if (IW::SW()->getWorkingUser()->isMaster()) {
            return true;
        }

        return false;
    }

    /**
     * Load backups.
     */
    public function indexAction()
    {
        $isAuthorized = $this->_Plugin->userIsAuthorized();

        if (!$isAuthorized) {
            $this->_displayFailure('_notAuthorized', self::AB_MESSAGE_NOT_AUTHORIZED);
        }

        $BackupAgent = $this->_Plugin->getBackupAgent();
        $backupAgentNotInstalled = !$BackupAgent->isInstalled();
        $scriptsMissing = $BackupAgent->scriptsMissing();

        if ($backupAgentNotInstalled || $scriptsMissing) {
            $View = $this->getView();
            $View->assign(self::CONTACT_ADMIN, true);
        }

        // js handles what to display by xhr requests if installed & authorized
    }

    /**
     * Exists because index action can't refer to itself
     * when displayFailure is triggered.
     */
    private function _notAuthorizedAction()
    {
        $View = $this->getView();
        $View->assign(self::IS_AUTHORIZED, false);
    }

    /**
     * Get backups.
     *
     * @param Form_SubmitOnly $Form
     */
    public function listBackupsCommitAction(Form_SubmitOnly $Form)
    {
        try {
            $backups = $this->_getBackups();
            $this->_displayPayloadAsJson($backups);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }


    /**
     * Get backups.
     *
     * @return array
     */
    private function _getBackups()
    {
        $backups = array();

        $CloudStorage = $this->_Plugin->getCloudStorage();
        if ($CloudStorage->isProtected()) {
            $Resource = $this->_Plugin->getResource();
            $backups = $Resource->getBackups();
            $returnValue = array_map(function ($backup) {
                return array(
                    'backupId' => $backup['backupId'],
                    'date' => $backup['date'],
                    'created' => $backup['created']);
            }, $backups);
        }

        return $returnValue;
    }


    /**
     * Get root content.
     *
     * @param Form_AcronisBackups $Form
     */
    public function getRootContentCommitAction(Form_AcronisBackups $Form)
    {
        try {
            $backupId = $Form->getBackupId();
            $RecoveryPoint = $this->_Plugin->getRecoveryPoint($backupId);
            $dirContent = $RecoveryPoint->getSwRootContent();

            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get directory content by id.
     *
     * @param Form_AcronisBackups_ListDirectoryContent $Form
     */
    public function getDirectoryContentCommitAction(Form_AcronisBackups_ListDirectoryContent $Form)
    {
        $backupId = $Form->getBackupId();
        $dirId = $Form->getDirectoryId();

        if (!$this->_dirPathsValid($backupId, array($dirId))) {
            $this->_displayFailure('index', self::ACTION_NOT_ALLOWED);
        } else {
            $this->_displayDirContent($backupId, $dirId);
        }
    }

    /**
     * Display directory content.
     *
     * @param string $backupId
     * @param string $dirId
     */
    private function _displayDirContent($backupId, $dirId)
    {
        try {
            $RecoveryPoint = $this->_Plugin->getRecoveryPoint($backupId);
            $dirContent = $RecoveryPoint->getDirectoryContent($dirId);
            $this->_displayPayloadAsJson($dirContent);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }


    /**
     * Recover directories.
     *
     * @param Form_AcronisBackups_RecoverDirectories $Form
     */
    public function recoverDirectoriesCommitAction(Form_AcronisBackups_RecoverDirectories $Form)
    {
        $backupId = $Form->getBackupId();
        $dirIds = $Form->getDirectoryIds();
        $shouldDeleteOlderFiles = $Form->deleteRequested();

        if (!$this->_dirPathsValid($backupId, $dirIds)) {
            $this->_displayFailure('index', self::ACTION_NOT_ALLOWED);
        } else {
            $this->_recoverDirectories($backupId, $dirIds, $shouldDeleteOlderFiles);
        }
    }

    /**
     * Recover directories.
     *
     * @param string  $backupId
     * @param array   $dirIds
     * @param boolean $shouldDeleteOlderFiles
     */
    private function _recoverDirectories($backupId, array $dirIds, $shouldDeleteOlderFiles = false)
    {
        try {
            $RecoveryPlan = $this->_Plugin->getRecoveryPlan($backupId);
            $RecoveryPlan->recoverDirectories($dirIds, $shouldDeleteOlderFiles);

            $Payload = new Payload(self::DIRECTORIES_RECOVERED);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Get restore location.
     */
    public function getRestoreAltLocationAction()
    {
        $altLocation = Model_RecoveryPlan_RecoveryPlan::getSwRestoreAltLocationPath();
        $Payload = new Payload(array(
            'altLocation' => $altLocation
        ));

        $this->_displayPayload($Payload);
    }



    /**
     * Restore directories to alt location.
     *
     * @param Form_AcronisBackups_RestoreAltLocation $Form
     */
    public function restoreToAltLocationCommitAction(Form_AcronisBackups_RestoreAltLocation $Form)
    {
        $backupId = $Form->getBackupId();
        $dirIds = $Form->getDirectoryIds();

        if (!$this->_dirPathsValid($backupId, $dirIds)) {
            $this->_displayFailure('index', self::ACTION_NOT_ALLOWED);
        } else {
            $this->_restoreToAltLocation($backupId, $dirIds);
        }
    }

    /**
     * Restore to alt location
     *
     * @param string $backupId
     * @param array  $dirIds
     */
    private function _restoreToAltLocation($backupId, array $dirIds)
    {
        try {
            $RecoveryPlan = $this->_Plugin->getRecoveryPlan($backupId);
            $RecoveryPlan->restoreToSwAltLocation($dirIds);

            $Payload = new Payload(self::RESTORE_COMPLETE_ALT_LOCATION);
            $this->_displayPayload($Payload);
        } catch (Exception $e) {
            $this->_logError(__FUNCTION__, $e);
            $this->_displayFailure('index', self::AB_MESSAGE_API_FAILURE);
        }
    }

    /**
     * Make sure that dirs are within sw root.
     *
     * @param string $backupId
     * @param array  $dirIds
     */
    private function _dirPathsValid($backupId, array $dirIds)
    {
        $RecoveryPoint = $this->_Plugin->getRecoveryPoint($backupId);
        foreach ($dirIds as $dirId) {
            if (!$RecoveryPoint->swDirPathValid($dirId)) {
                return false;
            }
        }

        return true;
    }

    /**
     * Display payload as json if CLI
     *
     * @param mixed $payloadContent
     */
    private function _displayPayloadAsJson($payloadContent)
    {
        // render as json
        if (IW::Env()->matches(IW::CLI)) {
            IW::Env()->setOutputStyle('pretty');
        }

        $Payload = new Payload($payloadContent);
        $this->_displayPayload($Payload);
    }

    /**
     * Log error msg.
     *
     * @param string    $functionName
     * @param Exception $e
     */
    private function _logError($functionName, Exception $e)
    {
        $exceptionDetails = $this->_getMessageFromException($e);
        IW::getLog()->error("Action failed $functionName: $exceptionDetails");
    }

    /**
     * Get message from exception
     *
     * @param Exception $e
     */
    private function _getMessageFromException(Exception $e)
    {
        $class = $e->getFile();
        $line = $e->getLine();
        $message = $e->getMessage();
        return "$message. File: $class, line: $line";
    }
}
