<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 20, 2020
 */

/**
 * Resource class.
 *
 * @package Interworx
 * @subpackage Plugin
 */
class Model_Resource_Resource extends Model_Acronis
{
    const NEXTBACKUP = 'nextBackup';
    const LASTBACKUP = 'lastBackup';
    const LASTSTATUS = 'lastStatus';
    const RUNNING = 'running';

    private $_instanceId;
    private $_resourceId;

    /**
     * Constructor.
     *
     * @param string $instanceId
     * @param string $resourceId
     */
    public function __construct($instanceId, $resourceId)
    {
        $this->_resourceId = $resourceId;
        $this->_instanceId = $instanceId;
    }


    /**
     * Get plan id.
     *
     * Will try to retrieve id in the following order:
     * cache > get assigned plan > assign existing plan
     *
     * @return string
     * @throws Exception // if no applicable plan exists.
     */
    public function getPlanId()
    {
        $Cache = $this->_getCache();
        $planId = $Cache->getPlanId();

        if (!isset($planId)) {
            $planId = $this->_getPlanIdFromApi();
            $Cache->cachePlanId($planId);
        }

        return $planId;
    }

    /**
     * Get plan id.
     * Checks for existing plans & creates a new one if needed.
     * Throws exception if no applicable plan could be added.
     *
     *
     * @return string
     */
    private function _getPlanIdFromApi()
    {
        try {
            $resourcePlanId = $this->_getAssignedPlanId();
            return $resourcePlanId;
        } catch (Exception $e) {
            $planId = $this->_addResourceToApplicablePlan();
            return $planId;
        }
    }

    /**
     * Get id from any of the already assigned plans.
     *
     * @return string
     */
    private function _getAssignedPlanId()
    {
        $Api = $this->_getApi();
        $resourcePlans = $Api->getBackupPlans($this->_instanceId);

        foreach ($resourcePlans as $plan) {
            $planId = $plan['planId'];
            $planOk = $this->_prePostScriptsOk($planId);

            if ($planOk) {
                return $planId;
            }
        }

        throw new Exception('no suitable plan is assigned to the resource.');
    }

    /**
     * Make sure pre/post scripts are enabled and configured.
     *
     * @param string $planId
     */
    protected function _prePostScriptsOk($planId)
    {
        $BackupPlan = $this->_getPlugin()->getBackupPlan($planId);
        return $BackupPlan->prePostScriptsOK();
    }

    /**
     * Add resource to a random applicable plan.
     */
    private function _addResourceToApplicablePlan()
    {
        $Api = $this->_getApi();
        $applicablePlans = $Api->getApplicablePlans($this->_instanceId)['Plans'];

        foreach ($applicablePlans as $plan) {
            $planId = $plan['ID'];

            if ($this->_prePostScriptsOk($planId)) {
                $this->_addResourceToPlan($planId);
                return $planId;
            }
        }

        throw new Exception('No applicable plan was found! Create a new one.');
    }

    /**
     * Check if prepost scripts are ok before assigning plan.
     *
     * @param string $planId
     */
    protected function _addResourceToPlan($planId)
    {
        $BackupPlan = $this->_getPlugin()->getBackupPlan($planId);
        $BackupPlan->addResource($this->_instanceId);
    }

    /**
     * Get resource backup status. includes last backup, next backup, last backup status.
     *
     * @return array
     */
    public function getBackupStatus()
    {
        $Api = $this->_getApi();
        $apiRes = $Api->getStatus($this->_instanceId);
        $status = $this->_formatBackupStatus($apiRes[0]);
        return $status;
    }

    /**
     * Get next backup.
     *
     *
     */
    public function backupNotScheduled()
    {
        $backupStatus = $this->getBackupStatus();
        $nextBackup = $backupStatus[self::NEXTBACKUP];
        return empty($nextBackup);
    }

    /**
     * check if backup in progress.
     */
    public function backupInProgress()
    {
        $backupStatus = $this->getBackupStatus();
        $lastStatus = $backupStatus[self::LASTSTATUS];
        return $lastStatus === self::RUNNING;
    }

    /**
     * Format backup status object.
     *
     * @param array $backupStatus
     * @return array
     */
    private function _formatBackupStatus(array $backupStatus)
    {
        return array(
            self::LASTSTATUS => $backupStatus['lastStatus'],
            self::LASTBACKUP => $this->_formatDate($backupStatus['lastStartTime']),
            self::NEXTBACKUP => $this->_formatDate($backupStatus['nextStartTime'])
        );
    }


    /**
     * Get resource id.
     *
     * @return string
     */
    public function getResourceId()
    {
        return $this->_resourceId;
    }

    /**
     * Get instance id.
     *
     * @return string
     */
    public function getInstanceId()
    {
        return $this->_instanceId;
    }

    /**
     * Get backups.
     */
    public function getBackups()
    {
        $backups = $this->_getBackupsFromApi();
        $this->_getCache()->cacheBackups($backups);
        return $backups;
    }

    /**
     * Get backups from api.
     *
     * @return array
     */
    private function _getBackupsFromApi()
    {
        $backups = $this->_getApi()->getBackups($this->_resourceId);
        $result = array();

        foreach ($backups as $backup) {
            $date = $this->_formatDate($backup['created']);

            $backupId = $backup['backup_id'];
            $legacyId = $backup['legacy_id'];
            $archive = $backup['archive'];

            $vaultId = $archive['vault_id'];
            $archiveId = $archive['id'];
            $machineId = $archive['agent_id'];

            $result[] = array(
                'date' => $date,
                'backupId' => $backupId,
                'legacyId' => $legacyId,
                'vaultId' => $vaultId,
                'archiveId' => $archiveId,
                'machineId' => $machineId,
                'created' => $backup['created'],
                'tenantAgentId' => $backup['tenant_id']
            );
        }

        return $result;
    }
}
