<?php
/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    Apr 28, 2020
 */

require_once(__DIR__ . '/Acronis.php');
require_once(__DIR__ . '/../../Model/CloudStorage/CloudStorage.php');
require_once(__DIR__ . '/../../Model/CloudStorage/Api.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class CloudStorageTest extends AcronisTest
{
    const RESOURCE_ID = 'resourceId';
    const INSTANCE_ID = 'instanceId';
    const LAST_BACKUP = 0;
    const NEXT_BACKUP = 0;
    const ONE_GB_IN_BYTES = 1073741824;
    const IS_ONLINE = true;

    private $_ApiMock;
    private $_SUT;

    private $_status;

    /**
     * Setup.
     */
    public function setUp()
    {
        $this->_setSutAndMock();
    }

    /**
     * set SUT and mock.
     */
    private function _setSutAndMock()
    {
        $this->_ApiMock = $this->_getApiMock();
        $this->_SUT = $this->_getSutPartialMock(Model_CloudStorage_CloudStorage::class, $this->_ApiMock);
    }

    /**
     * Get api mock
     */
    private function _getApiMock()
    {
        $Mock = $this->createMock(Model_CloudStorage_Api::class);

        $Mock->expects($this->any())
        ->method('getDevice')
        ->willReturn($this->_getDeviceInfo());

        $Mock->expects($this->any())
        ->method('getStorageData')
        ->willReturn(array(
           'occupied_data_size' => self::ONE_GB_IN_BYTES
        ));

        return $Mock;
    }

    /**
     * Get device info.
     */
    private function _getDeviceInfo()
    {
        return array(
            'id' => self::RESOURCE_ID,
            'lastBackup' => self::LAST_BACKUP,
            'nextBackup' => self::NEXT_BACKUP,
            'online' => self::IS_ONLINE,
            'instanceId' => self::INSTANCE_ID,
            'status' => isset($this->_status) ? $this->_status : 'ok'
        );
    }

    /**
     * test Get instance id
     */
    public function testGetInstanceId()
    {
        $actual = $this->_SUT->getInstanceId();
        $expected = self::INSTANCE_ID;

        $this->assertSame($expected, $actual);
    }

    /**
     * test get resource id
     */
    public function testGetResourceId()
    {
        $actual = $this->_SUT->getResourceId();
        $expected = self::RESOURCE_ID;

        $this->assertSame($expected, $actual);
    }

    /**
     * test get last backup
     */
    public function testGetLastBackup()
    {
        $actual = $this->_SUT->getLastBackup();
        $expected = self::LAST_BACKUP;

        $this->assertSame($expected, $actual);
    }

    /**
     * Test get next backup
     */
    public function testGetNextBackup()
    {
        $actual = $this->_SUT->getNextBackup();
        $expected  = self::NEXT_BACKUP;

        $this->assertSame($expected, $actual);
    }

    /**
     * test is online
     */
    public function testIsOnline()
    {
        $actual = $this->_SUT->isOnline();
        $expected = self::IS_ONLINE;

        $this->assertSame($expected, $actual);
    }

    /**
     * isProtected should return true.
     */
    public function testIsProtectedShouldReturnTrue()
    {
        $actual = $this->_SUT->isProtected();
        $this->assertTrue($actual);
    }


    /**
     * isProtected should return false when status = not_protected.
     */
    public function testIsProtectedShouldReturnFalse()
    {
        $this->_setStatus('not_protected');

        // retrigger setup with updated api mock.
        $this->_setSutAndMock();

        $actual = $this->_SUT->isProtected();
        $this->assertFalse($actual);
    }

    /**
     * getStatus should return ok.
     */
    public function testGetStatusShouldReturnOk()
    {
        $actual = $this->_SUT->getStatus();
        $expected = 'ok';

        $this->assertSame($expected, $actual);
    }

    /**
     * getStatus should return not_protected
     */
    public function testGetStatus()
    {
        $expected = 'not_protected';
        $this->_setStatus($expected);
        $this->_setSutAndMock();

        $actual = $this->_SUT->getStatus();
        $this->assertSame($expected, $actual);
    }

    /**
     * getStorageData should return storage size converted to GB.
     */
    public function testGetStorageData()
    {
        $result = $this->_SUT->getStorageData();
        $this->assertArrayHasKey('size', $result);

        $actual = $result['size'];
        $expected = '1 GB';
        $this->assertSame($expected, $actual);
    }

    /**
     * Set status.
     *
     * @param string $status
     */
    private function _setStatus($status)
    {
        $this->_status = $status;
    }
}
