<?php

/**
 * InterWorx Hosting Control Panel
 *
 * <pre>
 * +----------------------------------------------------------------------+
 * | Copyright (c) 2000-2010 InterWorx L.L.C., All Rights Reserved.       |
 * +----------------------------------------------------------------------+
 * | Redistribution and use in source form, with or without modification  |
 * | is NOT permitted without consent from the copyright holder.          |
 * |                                                                      |
 * | THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS "AS IS" AND |
 * | ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,    |
 * | THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A          |
 * | PARTICULAR PURPOSE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,    |
 * | EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,  |
 * | PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR   |
 * | PROFITS; OF BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY  |
 * | OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT         |
 * | (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE    |
 * | USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH     |
 * | DAMAGE.                                                              |
 * +----------------------------------------------------------------------+
 * </pre>
 *
 * @package InterWorx
 * @author  Malin Lorch <mlorch@liquidweb.com>
 * @date    May 04, 2020
 */

require_once(__DIR__ . '/Acronis.php');
require_once(__DIR__ . '/../../Model/RecoveryPoint/RecoveryPoint.php');
require_once(__DIR__ . '/../../Model/RecoveryPoint/Api.php');

/**
 *
 *
 * @package Interworx
 * @subpackage Plugin_Acronis
 */
class RecoveryPointTest extends AcronisTest
{
    const BACKUP_ID = 'somebackupid';

    const CACHE_CENTOS_ROOT = array('content' => 'cached centos root');
    const API_CENTOS_ROOT = array(
        'content' => array('id' => 'id1', 'displayName' => 'displayName1')
    );

    const CACHE_DIR = array('content' => 'cached centos root');
    const API_DIR = array(array('id' => 'id2', 'displayName' => 'displayName2'));

    const DIR_ID = 'somedirid';
    const LEGACY_ID = 'legacyid';
    const MACHINE_ID = 'machineid';
    const SUBSCRIPTION_ID = 'subid';

    private $_Api;
    private $_SUT;

    /**
     * Sut up. Runs before each test.
     */
    public function setUp()
    {
        $this->_Api = $this->_createMockApi();
        $this->_SUT = $this->_getSutPartialMock(Model_RecoveryPoint_RecoveryPoint::class, $this->_Api, self::BACKUP_ID);
    }

    /**
     * Create mock api.
     */
    private function _createMockApi()
    {
        $Mock = $this->createMock(Model_RecoveryPoint_Api::class);
        return $Mock;
    }

    /**
     * Test getDirectoryContent. SHould throw exception if cache does not contain backup.
     */
    public function testGetDirectoryContentShouldThrowExceptionIfCacheEmpty()
    {
        $this->expectException(Exception::class);
        $this->_SUT->getDirectoryContent();
    }

    /**
     * Expect getDirectoryContent to return cache if set.
     */
    public function testGetDirectoryContentWithoutDirIdShouldReturnCachedCentosDir()
    {
        $this->_setCacheBackup();
        $this->_formatCacheCentosRootReturnValue(self::CACHE_CENTOS_ROOT);

        $expected = self::CACHE_CENTOS_ROOT['content'];
        $actual = $this->_SUT->getDirectoryContent();
        $this->assertSame($expected, $actual);
    }

    /**
     * Format cache centos root return value.
     *
     * @param mixed $expected
     */
    private function _formatCacheCentosRootReturnValue($expected)
    {
        $Cache = $this->_getCacheMock();
        $Cache->expects($this->once())
            ->method('getCentosRoot')
            ->with(self::BACKUP_ID)
            ->willReturn($expected);
    }


    /**
     * Expect getDirectoryContent to return cache if set.
     */
    public function testGetDirectoryContentWithoutDirIdShouldReturnApiCentosDir()
    {
        $this->_setCacheBackup();
        $this->_formatApiGetCentosRoot();

        $actual = $this->_SUT->getDirectoryContent()[0];
        $expectedContent = self::API_CENTOS_ROOT['content'][0];

        $actualId = $actual['id'];
        $expectedId = $expectedContent['id'];
        $this->assertSame($expectedId, $actualId);


        $actualName = $actual['name'];
        $expectedName = $expectedContent['displayName'];
        $this->assertSame($expectedName, $actualName);
    }

    /**
     * format api res getCentosRoot
     */
    private function _formatApiGetCentosRoot()
    {
        $expectedCalls = 2;
        $this->_formatApiGetDir($expectedCalls, self::API_CENTOS_ROOT);
    }

    /**
     * test getDirectoryContent should return cache if any.
     */
    public function testGetDirContentWithIdShouldReturnCache()
    {
        $this->_formatCacheGetDirReturnValue(self::CACHE_DIR);
        $expected = self::CACHE_DIR['content'];
        $actual = $this->_SUT->getDirectoryContent(self::DIR_ID);

        $this->assertSame($expected, $actual);
    }

    /**
     * format cache getDir.
     *
     * @param string $expected
     */
    private function _formatcacheGetDirReturnValue($expected)
    {
        $this->_getCacheMock()->expects($this->once())
        ->method('getDirById')
        ->with(self::BACKUP_ID, self::DIR_ID)
        ->willReturn($expected);
    }

    /**
     * Test get dir content with id should return api res.
     */
    public function testGetDirContentWithIdShouldReturnApiRes()
    {
        $this->_setCacheBackup();
        $this->_formatApiGetDirWithId();

        $actual = $this->_SUT->getDirectoryContent(self::DIR_ID)[0];
        $expected = self::API_DIR[0];

        $expectedId = $expected['id'];
        $actualId = $actual['id'];
        $this->assertSame($expectedId, $actualId);


        $expectedName = $expected['displayName'];
        $actualName = $actual['name'];
        $this->assertSame($expectedName, $actualName);
    }

    /**
     * Format api getDir with id.
     */
    private function _formatApiGetDirWithId()
    {
        $expectedCalls = 1;
        $this->_formatApiGetDir($expectedCalls, self::API_DIR);
    }

    /**
     * Format getDir.
     *
     * @param integer $expectedCalls
     * @param mixed   $returnValue
     */
    private function _formatApiGetDir($expectedCalls, $returnValue)
    {
        $this->_Api->expects($this->exactly($expectedCalls))
        ->method('getDirectoryContent')
        ->willReturn($returnValue);
    }

    /**
     * Set cache return value getBackupById.
     */
    private function _setCacheBackup()
    {
        $this->_getCacheMock()->expects($this->any())
            ->method('getBackupById')
            ->willReturn(array(
                'legacyId' => self::LEGACY_ID,
                'machineId' => self::MACHINE_ID,
                'subscriptionId' => self::SUBSCRIPTION_ID
            ));
    }
}
